import { Alert } from "@mui/material";
import { ApiError, ErrorMessage } from "../../../types/common";
import { FetchBaseQueryError } from "@reduxjs/toolkit/query";
import { SerializedError } from "@reduxjs/toolkit";

interface ApiAlertErrorProps {
  error: ApiError;
}

export const isFetchBaseQueryError = (
  error: unknown
): error is FetchBaseQueryError => {
  return typeof error === "object" && error != null && "status" in error;
};

export const isErrorWithMessage = (
  error: unknown
): error is { message: string } => {
  return (
    typeof error === "object" &&
    error != null &&
    "message" in error &&
    typeof (error as SerializedError).message === "string"
  );
};

export const getErrorMessage = (
  error: FetchBaseQueryError | SerializedError | unknown
) => {
  if (error) {
    if (isFetchBaseQueryError(error)) {
      const customError = error.data as ErrorMessage;
      if (customError) {
        return customError.description
          ? customError.description
          : customError.error;
      }
      return "Unknown error";
    } else if (isErrorWithMessage(error)) {
      return error.message;
    }
  }
  return "Unknown error";
};

export const ApiAlertError = ({ error }: ApiAlertErrorProps) => (
  <Alert severity="error">{getErrorMessage(error)}</Alert>
);
