import {
  useState,
  useCallback,
  useRef,
  useEffect,
  useContext,
  useMemo
} from "react";
import { apiContext } from "../../api/DeployApi";
import { useAPIMethod, usePolling } from "@ess-ics/ce-ui-common";
import { LogStreamConsole } from "../common/LogStream/LogStreamConsole";
import { LogStreamConsoleDialog } from "../common/LogStream/LogStreamConsoleDialog";
import { PopoutButton } from "../common/Buttons/PopoutButton";
import { LinearProgress, Stack } from "@mui/material";

const LOG_POLL_INTERVAL = 5000;

export function DeploymentJobOutput({ deploymentJob }) {
  const [consoleDialogOpen, setConsoleDialogOpen] = useState(false);
  const client = useContext(apiContext);
  const finalResultsNeeded = useRef(true);
  const params = useMemo(
    () => ({
      job_id: deploymentJob?.id
    }),
    [deploymentJob]
  );

  const {
    value: log,
    wrapper: getLogById,
    loading: logLoading,
    dataReady: logDataReady,
    abort: abortGetLogById
  } = useAPIMethod({
    fcn: client.apis.Operations.fetchDeploymentJobLog,
    params
  });

  const getLog = useCallback(() => {
    if (!deploymentJob.finished || finalResultsNeeded.current) {
      getLogById(deploymentJob.id);
      finalResultsNeeded.current = !deploymentJob.finished;
    }
  }, [deploymentJob.finished, deploymentJob.id, getLogById]);

  usePolling(
    getLog,
    logLoading || !logDataReady,
    LOG_POLL_INTERVAL,
    abortGetLogById
  );

  useEffect(() => {
    finalResultsNeeded.current = true;
  }, [deploymentJob.id]);

  return (
    <Stack>
      <Stack
        display="flex"
        flexDirection="row"
        justifyContent="flex-end"
        sx={{ marginBottom: (theme) => theme.spacing(2) }}
      >
        <PopoutButton
          title="Popout log"
          onButtonClick={() => setConsoleDialogOpen(true)}
        />
      </Stack>
      <LogStreamConsoleDialog
        title="Job log stream"
        loading={!log && !deploymentJob?.started}
        open={consoleDialogOpen}
        log={log?.stdoutHtml}
        dataReady={logDataReady}
        onDialogClose={() => setConsoleDialogOpen(false)}
      />
      {log && deploymentJob?.started ? (
        <LogStreamConsole
          log={log.stdoutHtml}
          dataReady={logDataReady}
          height="500px"
        />
      ) : (
        <div style={{ width: "100%" }}>
          <LinearProgress color="primary" />
        </div>
      )}
    </Stack>
  );
}
